/* 
 * POSE DB Export
 * 
 * schuhmac@uni-duesseldorf.de
 * 
 */

#include <Common.h>
#include <System/SysAll.h>
#include <UI/UIAll.h>
#include <HostControl.h>
#include "PoseDBExp.h"
#include "PoseDBExpIDs.h"

#define AppID    'PdbE'


/*****************************************************************************/

int numDB;
CharPtr (*ppDBNames);
ListPtr lstDBPtr;

const char defaultpath[] = "c:\\";

static int StartApplication(void);
static void EventLoop(void);
static void StopApplication(void);
static Boolean frmMainFormEH(EventPtr event);
static void SetupDatabaseList(void);
static int MySortCompareFunc(CharPtr *s1, CharPtr *s2, long other);

static VoidPtr GetObjectPtr(Word objID);

/*****************************************************************************/

static VoidPtr GetObjectPtr(Word objID)
{
    FormPtr form=FrmGetActiveForm();
	return FrmGetObjectPtr(form, FrmGetObjectIndex(form, objID));
}

DWord  PilotMain (Word cmd, Ptr cmdPBP, Word launchFlags)
{
  int error;
  if (cmd == sysAppLaunchCmdNormalLaunch)
  {
    error = StartApplication();  // Application start code
    if (error) return error;
    EventLoop();                 // Event loop
    StopApplication ();          // Application stop code
  }
  return 0;
}

static int StartApplication(void)
{
  FrmGotoForm(frmMainForm);
  return 0;
}

static void EventLoop(void)
{
  short err;
  int formID;
  FormPtr form;
  EventType event;

  do
  {
    EvtGetEvent(&event, -1);

    if (SysHandleEvent(&event)) continue;
    if (MenuHandleEvent((void *)0, &event, &err)) continue;

    if (event.eType == frmLoadEvent)
    {
      formID = event.data.frmLoad.formID;
      form = FrmInitForm(formID);
      FrmSetActiveForm(form);
      switch (formID)
      {
      case frmMainForm:
	    FrmSetEventHandler(form, (FormEventHandlerPtr) frmMainFormEH);
	    break;
	  default:
	    break;  
      }
    }
    FrmDispatchEvent(&event);
  }
  while(event.eType != appStopEvent);
}

static void StopApplication(void)
{
  FrmSaveAllForms ();
  FrmCloseAllForms ();
}

static Boolean frmMainFormEH(EventPtr event)
{
  FormPtr   form;
  Boolean   handled = false;
  CharPtr   dbname, errstr;
  HostErr   err;
  Char      savename[255];
  FieldPtr  pathfieldptr;
  int       len;
  VoidHand  memh;

  switch (event->eType)
  {
    case frmOpenEvent:
      form = FrmGetActiveForm();
      SetupDatabaseList();
      lstDBPtr = GetObjectPtr(lstDB);
      LstSetListChoices(lstDBPtr, ppDBNames, numDB);
      len = sizeof(defaultpath);
      memh = MemHandleNew(len+1);
      StrCopy(MemHandleLock(memh), defaultpath);
      MemHandleUnlock(memh);
      FldSetText(GetObjectPtr(fldPath), memh, 0, len);
      FldSetInsPtPosition(GetObjectPtr(fldPath), len);
      FrmDrawForm(form);
      FrmSetFocus(form, FrmGetObjectIndex(form, fldPath));
      handled = true;
      break;
    case ctlSelectEvent:
      switch (event->data.ctlEnter.controlID) 
      {
        case cmdSave:
          dbname = ppDBNames[LstGetSelection(lstDBPtr)];
          if (!HostIsSelectorImplemented(hostSelectorExportFile))
          {
            FrmAlert(altNotImplemented);
          }  
          else
          { 
            form = FrmGetActiveForm();
            pathfieldptr = FrmGetObjectPtr(form, FrmGetObjectIndex(form, fldPath));
            if (FldGetTextPtr(pathfieldptr) != NULL)
            {
              StrCopy(savename, FldGetTextPtr(pathfieldptr));
            }
            if (savename[StrLen(savename)-1] != '\\')
            {
              StrCat(savename, "\\");
            }  
            StrCat(savename, dbname);
            StrCat(savename, ".prc");
            err = HostExportFile(savename, 0, dbname);
            if (err != 0)
            {
              errstr = MemPtrNew (20);
              StrIToA(errstr, err);
              FrmCustomAlert(altErrorSaving, errstr, NULL, NULL);
              MemPtrFree (errstr);            
            }
            else
            {   
              FrmCustomAlert(altSavingDB, dbname, savename, NULL);
            }
          }  
          handled = true;
          break;
        case cmdAbout:
          FrmAlert(altAbout);
          handled = true;
          break;
      }
      break;      
   default:
	  break;  
  }
  return handled;
}

static void SetupDatabaseList(void)
{
  int i, len;
  CharPtr dbname;
  LocalID dbid;
  char name[33];
  
  numDB = DmNumDatabases(0);
  ppDBNames = MemPtrNew(numDB*sizeof(CharPtr));
  
  for (i=0; i<numDB; i++)
  {
    dbid = DmGetDatabase(0, i);
    DmDatabaseInfo(0, dbid, name, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
    len = StrLen(name);
    dbname = MemPtrNew(len+1);
    StrCopy(dbname, name);
    ppDBNames[i] = dbname;
  }
  SysQSort(ppDBNames, numDB, sizeof(CharPtr), (CmpFuncPtr)MySortCompareFunc, 0);
}

static int MySortCompareFunc(CharPtr *s1, CharPtr *s2, long other)
{
  return StrCompare(*s1, *s2);
}